/**
*	WARNING: Do Not Remove This Section
*
*	$RCSfile: bitset.h,v $
*	$Revision: 1.8.2.1 $
*	$Date: 2007/12/21 17:36:16 $
*	$Author: d.jansen $
*
*	MRMC is a model checker for discrete-time and continuous-time Markov
*	reward models. It supports reward extensions of PCTL and CSL (PRCTL
*	and CSRL), and allows for the automated verification of properties
*	concerning long-run and instantaneous rewards as well as cumulative
*	rewards.
*
* 	Copyright (C) The University of Twente, 2004-2008.
* 	Copyright (C) RWTH Aachen, 2008-2009.
*	Authors: Maneesh Khattri, Ivan Zapreev, David N. Jansen
*
*	This program is free software; you can redistribute it and/or
*	modify it under the terms of the GNU General Public License
*	as published by the Free Software Foundation; either version 2
*	of the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program; if not, write to the Free Software
*	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
*	MA  02110-1301, USA.
*
*	Main contact:
*		Lehrstuhl für Informatik 2, RWTH Aachen University
*		Ahornstrasse 55, 52074 Aachen, Germany
*		E-mail: info@mrmc-tool.org
*
*	Old ontact:
*		Formal Methods and Tools Group, University of Twente,
*		P.O. Box 217, 7500 AE Enschede, The Netherlands,
*		Phone: +31 53 4893767, Fax: +31 53 4893247,
*		E-mail: mrmc@cs.utwente.nl
*
*	Source description: Manage a Bitset - For the satisfaction relation
*	in MCC.
*	Uses: Definition of bitset - bitset.h
*/

#ifndef BITSET_H
#define BITSET_H

#include "macro.h"

/* The following declarations may be included in macro.h (or a special file
   error.h) later: */

	/**************************************/
	/* Special Types for Use with SPLINT  */
	/**************************************/

	/* Splint checks whether type usage is consistent. This reduces the
	danger to mix several uses of ``int''.
	The types below should be used in a way that they could be either
	unsigned or signed; for example, the value ``-1'' should be avoided. */

	typedef int state_index; /* index of a state */
	typedef int state_count; /* number of states */
	#define state_index_NONE ((state_index) -1) /* indicates `not found' */
	#define state_index_ERROR ((state_index) -2) /* indicates an error */
	#define state_count_NONE ((state_count) -1)
	#define state_count_ERROR ((state_count) -2)

	/**************************************/
	/*********** Error checking ***********/
	/**************************************/

	typedef enum { err_OK, err_ERROR } err_state;
	#define err_PARAM "Error: illegal parameter in "
	#define err_MEMORY "Error: out of memory in "
	#define err_CALLBY "called by "

	extern BOOL err_state_iserror(err_state a) /*@*/;
	#define err_state_iserror(a) (err_OK != (a))

	/**
	* These macros handles an error. They print an error message and return.
	* err_macro_<number> is meant for a macro with <number> parameters;
	* the retval is the result of the macro.
	* err_msg_<number> is meant to be used as a statement in a function;
	* the macro includes an explicit ``return'' statement.
	* @param type type of the error, e. g. err_PARAM, err_MEMORY
	* @param func name (with printf-spec of the parameters) of the function
	* @param p1,p2,... parameters of the function. The number of parameters
	*		must fit the printf-spec
	* @param retval value to be returned to the caller
	*/
	/*@notfunction@*/
	#define err_macro_0(type,func,retval) \
			(fprintf(stderr, "%s%s(%u) " func "\n", (type), \
				__FILE__, __LINE__), (retval))

	/*@notfunction@*/
	#define err_macro_1(type,func,p1,retval) \
			(fprintf(stderr, "%s%s(%u) " func "\n", (type), \
				__FILE__, __LINE__, (p1)), (retval))

	/*@notfunction@*/
	#define err_macro_2(type,func,p1,p2,retval) \
			(fprintf(stderr, "%s%s(%u) " func "\n", (type), \
				__FILE__, __LINE__, (p1), (p2)), (retval))

	/*@notfunction@*/
	#define err_macro_3(type,func,p1,p2,p3,retval) \
			(fprintf(stderr, "%s%s(%u) " func "\n", (type), \
				__FILE__, __LINE__, (p1), (p2), (p3)), (retval))

	/*@notfunction@*/
	#define err_macro_4(type,func,p1,p2,p3,p4,retval) \
			(fprintf(stderr, "%s%s(%u) " func "\n", (type), \
				__FILE__, __LINE__, (p1), (p2), (p3), (p4)), \
			 (retval))

	/*@notfunction@*/
	#define err_macro_5(type,func,p1,p2,p3,p4,p5,retval) \
			(fprintf(stderr, "%s%s(%u) " func "\n", (type), \
				__FILE__, __LINE__, (p1),(p2),(p3),(p4),(p5)), \
			 (retval))

	/*@notfunction@*/
	#define err_msg_0(type,func,retval) \
			{ return err_macro_0((type), func, (retval)); }
	/*@notfunction@*/
	#define err_msg_1(type,func,p1,retval) \
			{ return err_macro_1((type), func, (p1), (retval)); }
	/*@notfunction@*/
	#define err_msg_2(type,func,p1,p2,retval) \
			{ return err_macro_2((type),func,(p1),(p2),(retval)); }
	/*@notfunction@*/
	#define err_msg_3(type,func,p1,p2,p3,retval) \
			{ return err_macro_3((type), func, (p1), (p2), (p3), \
				(retval)); }
	/*@notfunction@*/
	#define err_msg_4(type,func,p1,p2,p3,p4,retval) \
			{ return err_macro_4((type), func, (p1), (p2), (p3), \
				(p4), (retval)); }
	/*@notfunction@*/
	#define err_msg_5(type,func,p1,p2,p3,p4,p5,retval) \
			{ return err_macro_5((type), func, (p1), (p2), (p3), \
				(p4), (p5), (retval)); }
	/*@notfunction@*/
	#define err_msg_va(type,func,p1,retval) \
			{ \
				va_list m_va; \
				fprintf("%s%s(%u) ",(type),__FILE__,__LINE__); \
				va_start(m_va, (p1)); \
				vfprintf(stderr, func "\n", m_va); \
				va_end(m_va); \
				return (retval); \
			}


#include <math.h>

/* This is the type of one block in the bitset */
typedef unsigned int BITSET_BLOCK_TYPE;
/* This is the size of the bitset block */
#define BITSET_BLOCK_SIZE (sizeof(BITSET_BLOCK_TYPE) * 8)

/**
*This are the constant bits
*/
#define BIT_ON	((BITSET_BLOCK_TYPE) 1)
#define BIT_OFF	((BITSET_BLOCK_TYPE) 0)

/**
* The bitset structure
* @member n			: The number of bits in this bitset.
* @member bytesp		: An array of BITSET_BLOCK_TYPE.
*/
typedef /*@abstract@*/ /*@immutable@*/ struct bitset
{
	state_count n;
	BITSET_BLOCK_TYPE bytesp[1];
} bitset;

/**
* Get a new bitset.
* A bitset is composed of an array of BITSET_BLOCK_SIZE-bit unsigned integers.
* @param n indicates the size of the required bitset.
* @return returns a pointer to a new bitset. NULL if some error has occurred.
*/
extern /*@null@*/ /*@only@*/ bitset * get_new_bitset(state_count n)
		/*@modifies internalState@*/;

/**
* This function extends a bitset, similar to realloc().
* We assume that the bitset size may only be increased.
* @param pBitset the bitset to increase
* @param new_size the new bitset size, should be: "new_size >= pBitset->n"
* @param isFillNewBlocksWithOne if TRUE then the new bits will be set.
* @result the new address of the bitset. If some error occurred, the function
*		returns NULL and does not change anything. In particular, the
*		old bitset is still allocated.
* (former function name: extend_bitset)
*/
extern /*@null@*/ /*@only@*/ bitset * bs_extend(
		/*@null@*/ /*@only@*/ bitset * pBitset,
		state_count new_size, BOOL isFillNewBlocksWithOne)
		/*@modifies internalState, *pBitset@*/;

/**
* This function frees a bitset.
* @param a: the bitset that has to be freed.
* The suffix ``_noerror'' indicates that this function never returns an error.
* Therefore, it is not necessary to test a return value.
*/

/* prototype for macro: helps splint to check for correct usage */
INLINE void free_bitset(/*@only@*/ /*@out@*/ bitset * a) /*@modifies internalState@*/ {
	IF_SAFETY( a != NULL )
		free(a);
	ELSE_SAFETY
		printf("ERROR: Trying to deallocate a bitset pointed by NULL.\n");
		exit(1);
	ENDIF_SAFETY
}

/**
* This method just copies elements from one bitset into another.
* @param from_bitset the bitset to be copied .
* @param to_bitset the bitset to copy to.
* NOTE: The two bitsets must have the same size.
*/
extern err_state copy_bitset(const bitset *from_bitset, bitset *to_bitset)
		/*@modifies *to_bitset@*/;

/**
* Allocate a bitset and set it to the disjunction of two bitsets.
* @param a one of the operands to the disjunction operation.
* @param b the second operand to the disjuction operation.
* @return returns a pointer to a new bitset that contains the result of
*		disjunction. NULL if some error occurred, e. g. a and b have
*		different sizes.
*/
extern /*@null@*/ /*@only@*/ bitset * or(const bitset *a, const bitset *b)
		/*@modifies internalState@*/;

/**
* Obtain the disjunction of two bitsets and put result in arg2.
* @param a one of the operands to the disjunction operation.
* @param b the second operand to the disjunction operation. Upon return, it will
*		contain the disjunction.
*/
extern err_state or_result(const bitset *a, bitset *reslt)
		/*@modifies *reslt@*/;

/**
* Allocate a bitset and set it to the exclusive-or of two bitsets.
* @param a one of the operands to the exclusive-or operation.
* @param b the second operand to the exclusive-or operation.
* @return returns a pointer to a new bitset that contains the result of
*		exclusive-or. NULL if some error occurred, e. g. a and b have
*		different sizes.
*/
extern /*@null@*/ /*@only@*/ bitset * xor(const bitset * a, const bitset * b)
		/*@modifies internalState@*/;

/**
* Obtain the exclusive-or of two bitsets and put result in arg2.
* @param a one of the operands to the exclusive-or operation.
* @param b the second operand to the exclusive-or operation. Upon return, it
*		will contain the exclusive-or.
*/
extern err_state xor_result(const bitset *a, bitset * reslt)
		/*@modifies *reslt@*/;

/**
* Allocate a bitset and set it to the conjunction of two bitsets.
* @param a one of the operands to the conjunction operation.
* @param b the second operand to the conjuction operation.
* @return returns a pointer to a new bitset that contains the result of
*		conjunction. NULL if some error occurred, e. g. a and b have
*		different sizes.
*/
extern /*@null@*/ /*@only@*/ bitset * and(const bitset * a, const bitset * b)
		/*@modifies internalState@*/;

/**
* Obtain the conjunction of two bitsets and put result in arg2.
* @param a one of the operands to the conjunction operation.
* @param b the second operand to the conjunction operation. Upon return, it will
*		contain the conjunction.
*/
extern err_state and_result(const bitset *a, bitset *reslt)
		/*@modifies *reslt@*/;

/**
* Allocate a bitset and set it to the complement of a bitset.
* @param a the operand to the complement operation.
* @return returns a pointer to a new bitset that contains the result of
*		complementation. NULL if some error occurred.
*/
extern /*@null@*/ /*@only@*/ bitset * not(const bitset * a)
		/*@modifies internalState@*/;

/**
* Obtain the complement of a bitsets and put result in the argument.
* @param reslt the operand to the complement operation. Upon return, it will
*		contain the complement.
*/
extern err_state not_result(bitset * reslt)
		/*@modifies *reslt@*/;

/**
* Gets the value of a specified bit of a bitset.
* @param a One of the bits of this bitset is of interest.
* @param i The position of the bit of interest in the bitset.
* @return TRUE if the bit is set to 1; FALSE otherwise; -1 if some error
*		occurred.
*/

/* prototype for macro: helps splint to check for correct usage */
extern BOOL get_bit_val(/*@sef@*/ const bitset * a, /*@sef@*/ state_index i)
		/*@modifies nothing@*/;
#define get_bit_val(a,i) \
		(NULL == (a) || (unsigned) (i) >= (unsigned) (a)->n \
		 ? err_macro_3(err_PARAM, "get_bit_val(%p[%d],%d)", \
				(const void *)(a), NULL != (a) ? (a)->n : 0, \
				(i), -1) \
		 : BIT_OFF != ((a)->bytesp[(i) / BITSET_BLOCK_SIZE] \
			       & ((BITSET_BLOCK_TYPE) 1 \
				  << ((unsigned) (i) % BITSET_BLOCK_SIZE))))

/**
* Sets the value of a specified bit of a bitset.
* @param a One of the bits of this bitset is to be changed.
* @param i The position of the bit of interest in the bitset.
* @param val The new value for the bit of interest: 0 = clear bit; other value =
*		set bit.
*/

/* prototype for macro: helps splint to check for correct usage */
extern err_state set_bit_val(/*@sef@*/ bitset * a, /*@sef@*/ state_index i,
		/*@sef@*/ BITSET_BLOCK_TYPE val) /*@modifies *a@*/;
#define set_bit_val(a,i,val) \
		(NULL == (a) || (unsigned) (i) >= (unsigned) (a)->n \
		 ? err_macro_4(err_PARAM, "set_bit_val(%p[%d],%d,%u)", \
				(void *) (a), NULL != (a) ? (a)->n : 0, (i), \
				(val), err_ERROR) \
		 : (BIT_OFF != (val) \
		    ? (void) ((a)->bytesp[(i) / BITSET_BLOCK_SIZE] |= \
			(BITSET_BLOCK_TYPE) 1 \
				<< ((unsigned) (i) % BITSET_BLOCK_SIZE)) \
		    : (void) ((a)->bytesp[(i) / BITSET_BLOCK_SIZE] &= \
			~((BITSET_BLOCK_TYPE) 1 \
				<< ((unsigned) (i) % BITSET_BLOCK_SIZE))), \
		    err_OK))

/**
* Print the position of the bits in the given bitset whose value
* is 1.
* @param a the bitset to be printed.
*/
extern err_state print_bitset_states(const bitset *)
		/*@modifies fileSystem@*/;

/**
* Fills the given bitset with one.
* @param a the bitset to be filled.
* @return returns a pointer to the result of the filling operation.
*/
extern err_state fill_bitset_one(bitset * a) /*@modifies *a@*/;

/**
* Fills the given bitset with zero.
* @param a the bitset to be filled.
* @return returns a pointer to the result of the filling operation.
*/
extern err_state fill_bitset_zero(bitset * a) /*@modifies *a@*/;

/**
* Checks if the bitset contains only zeros.
* @param a the bitset to be checked.
* @return 0: bitset contains a non-zero, 1: bitset contains only zeros.
*/
extern BOOL is_bitset_zero(const bitset *) /*@modifies nothing@*/;

/**
* Get the Index of the next non-zero element.
* @param a the bitset to be checked.
* @param idx the Present index. If set to -1 then it is the first time call
*		for this method on the given bitset i.e. we have
*		to start searching from the very beginning
* @return the next index, or -1 if there are no more elements left
*/
extern state_index get_idx_next_non_zero(const bitset *, state_index)
		/*@modifies nothing@*/;

/**
* Count the number of non-zero elements in the given bitset.
* @param a the bitset to be checked.
* @return the count.
*/
extern state_count count_non_zero(const bitset *) /*@modifies nothing@*/;

/**
* Allocate an array that contains the numbers of all bits that are set in the
* bitset.
* the first element of the array is the count.
* the rest are the bit ids of set bits.
* @param	: bitset *toCount: the bitset in question
* @return	: the count of bits that are set and their ids.
*/
extern /*@only@*/ /*@null@*/ state_index * count_set(
		const bitset *toCount) /*@modifies internalState@*/;

#endif
